@@ -1,8 +1,5 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE DefaultSignatures #-}
-#if __GLASGOW_HASKELL__ >= 800
-{-# OPTIONS_GHC -fno-warn-redundant-constraints #-}
-#endif
 
 -----------------------------------------------------------------------------
 -- |
@@ -25,6 +22,10 @@ import Data.Int
 import Data.Word
 import Foreign.C.Types (CFloat, CDouble)
 
+-- $setup
+-- >>> import Data.Complex (Complex (..))
+
+
 -- | An involutive ring
 class Num a => Conjugate a where
   -- | Conjugate a value. This defaults to the trivial involution.
